@@ -6,6 +6,7 @@ import json
 import os
 
 from dataclasses import dataclass
+from markdownify import markdownify as md
 from pathlib import Path
 from sqlcipher3 import dbapi2 as sqlite
 from typing import List, Sequence, Tuple
@@ -18,6 +19,8 @@ class DBConfig:
     path: Path
     key: str
     idle_minutes: int = 15  # 0 = never lock
+    theme: str = "system"
+    move_todos: bool = False
 
 
 class DBManager:
@@ -159,13 +162,6 @@ class DBManager:
         ).fetchone()
         return row[0] if row else ""
 
-    def upsert_entry(self, date_iso: str, content: str) -> None:
-        """
-        Insert or update an entry.
-        """
-        # Make a new version and set it as current
-        self.save_new_version(date_iso, content, note=None, set_current=True)
-
     def search_entries(self, text: str) -> list[str]:
         """
         Search for entries by term. This only works against the latest
@@ -261,68 +257,31 @@ class DBManager:
         ).fetchall()
         return [dict(r) for r in rows]
 
-    def get_version(
-        self,
-        *,
-        date_iso: str | None = None,
-        version_no: int | None = None,
-        version_id: int | None = None,
-    ) -> dict | None:
+    def get_version(self, *, version_id: int) -> dict | None:
         """
-        Fetch a specific version by (date, version_no) OR by version_id.
+        Fetch a specific version by version_id.
         Returns a dict with keys: id, date, version_no, created_at, note, content.
         """
         cur = self.conn.cursor()
-        if version_id is not None:
-            row = cur.execute(
-                "SELECT id, date, version_no, created_at, note, content "
-                "FROM versions WHERE id=?;",
-                (version_id,),
-            ).fetchone()
-        else:
-            if date_iso is None or version_no is None:
-                raise ValueError(
-                    "Provide either version_id OR (date_iso and version_no)"
-                )
-            row = cur.execute(
-                "SELECT id, date, version_no, created_at, note, content "
-                "FROM versions WHERE date=? AND version_no=?;",
-                (date_iso, version_no),
-            ).fetchone()
+        row = cur.execute(
+            "SELECT id, date, version_no, created_at, note, content "
+            "FROM versions WHERE id=?;",
+            (version_id,),
+        ).fetchone()
         return dict(row) if row else None
 
-    def revert_to_version(
-        self,
-        date_iso: str,
-        *,
-        version_no: int | None = None,
-        version_id: int | None = None,
-    ) -> None:
+    def revert_to_version(self, date_iso: str, version_id: int) -> None:
         """
         Point the page head (pages.current_version_id) to an existing version.
-        Fast revert: no content is rewritten.
         """
-        if self.conn is None:
-            raise RuntimeError("Database is not connected")
         cur = self.conn.cursor()
 
-        if version_id is None:
-            if version_no is None:
-                raise ValueError("Provide version_no or version_id")
-            row = cur.execute(
-                "SELECT id FROM versions WHERE date=? AND version_no=?;",
-                (date_iso, version_no),
-            ).fetchone()
-            if row is None:
-                raise ValueError("Version not found for this date")
-            version_id = int(row["id"])
-        else:
-            # Ensure that version_id belongs to the given date
-            row = cur.execute(
-                "SELECT date FROM versions WHERE id=?;", (version_id,)
-            ).fetchone()
-            if row is None or row["date"] != date_iso:
-                raise ValueError("version_id does not belong to the given date")
+        # Ensure that version_id belongs to the given date
+        row = cur.execute(
+            "SELECT date FROM versions WHERE id=?;", (version_id,)
+        ).fetchone()
+        if row is None or row["date"] != date_iso:
+            raise ValueError("version_id does not belong to the given date")
 
         with self.conn:
             cur.execute(
@@ -346,20 +305,18 @@ class DBManager:
         ).fetchall()
         return [(r[0], r[1]) for r in rows]
 
-    def export_json(
-        self, entries: Sequence[Entry], file_path: str, pretty: bool = True
-    ) -> None:
+    def export_json(self, entries: Sequence[Entry], file_path: str) -> None:
         """
         Export to json.
         """
         data = [{"date": d, "content": c} for d, c in entries]
         with open(file_path, "w", encoding="utf-8") as f:
-            if pretty:
-                json.dump(data, f, ensure_ascii=False, indent=2)
-            else:
-                json.dump(data, f, ensure_ascii=False, separators=(",", ":"))
+            json.dump(data, f, ensure_ascii=False, indent=2)
 
     def export_csv(self, entries: Sequence[Entry], file_path: str) -> None:
+        """
+        Export pages to CSV.
+        """
         # utf-8-sig adds a BOM so Excel opens as UTF-8 by default.
         with open(file_path, "w", encoding="utf-8-sig", newline="") as f:
             writer = csv.writer(f)
@@ -373,6 +330,10 @@ class DBManager:
         separator: str = "\n\n— — — — —\n\n",
         strip_html: bool = True,
     ) -> None:
+        """
+        Strip the HTML from the latest version of the pages
+        and save to a text file.
+        """
         import re, html as _html
 
         # Precompiled patterns
@@ -411,6 +372,9 @@ class DBManager:
     def export_html(
         self, entries: Sequence[Entry], file_path: str, title: str = "Bouquin export"
     ) -> None:
+        """
+        Export to HTML with a heading.
+        """
         parts = [
             "<!doctype html>",
             '<html lang="en">',
@@ -430,6 +394,33 @@ class DBManager:
         with open(file_path, "w", encoding="utf-8") as f:
             f.write("\n".join(parts))
 
+    def export_markdown(
+        self, entries: Sequence[Entry], file_path: str, title: str = "Bouquin export"
+    ) -> None:
+        """
+        Export to HTML, similar to export_html, but then convert to Markdown
+        using markdownify, and finally save to file.
+        """
+        parts = [
+            "<!doctype html>",
+            '<html lang="en">',
+            "<body>",
+            f"<h1>{html.escape(title)}</h1>",
+        ]
+        for d, c in entries:
+            parts.append(
+                f"<article><header><time>{html.escape(d)}</time></header><section>{c}</section></article>"
+            )
+        parts.append("</body></html>")
+
+        # Convert html to markdown
+        md_items = []
+        for item in parts:
+            md_items.append(md(item, heading_style="ATX"))
+
+        with open(file_path, "w", encoding="utf-8") as f:
+            f.write("\n".join(md_items))
+
     def export_sql(self, file_path: str) -> None:
         """
         Exports the encrypted database as plaintext SQL.
@@ -450,6 +441,10 @@ class DBManager:
         cur.execute("DETACH DATABASE backup")
 
     def export_by_extension(self, file_path: str) -> None:
+        """
+        Fallback catch-all that runs one of the above functions based on
+        the extension of the file name that was chosen by the user.
+        """
         entries = self.get_all_entries()
         ext = os.path.splitext(file_path)[1].lower()
 
@@ -461,6 +456,10 @@ class DBManager:
             self.export_txt(entries, file_path)
         elif ext in {".html", ".htm"}:
             self.export_html(entries, file_path)
+        elif ext in {".sql", ".sqlite"}:
+            self.export_sql(file_path)
+        elif ext == ".md":
+            self.export_markdown(entries, file_path)
         else:
             raise ValueError(f"Unsupported extension: {ext}")
 
