@@ -10,6 +10,7 @@ from PySide6.QtGui import (
     QFontDatabase,
     QImage,
     QImageReader,
+    QPalette,
     QPixmap,
     QTextCharFormat,
     QTextCursor,
@@ -28,8 +29,11 @@ from PySide6.QtCore import (
     QBuffer,
     QByteArray,
     QIODevice,
+    QTimer,
 )
-from PySide6.QtWidgets import QTextEdit
+from PySide6.QtWidgets import QTextEdit, QApplication
+
+from .theme import Theme, ThemeManager
 
 
 class Editor(QTextEdit):
@@ -41,8 +45,13 @@ class Editor(QTextEdit):
     _HEADING_SIZES = (24.0, 18.0, 14.0)
     _IMAGE_EXTS = (".png", ".jpg", ".jpeg", ".bmp", ".gif", ".webp")
     _DATA_IMG_RX = re.compile(r'src=["\']data:image/[^;]+;base64,([^"\']+)["\']', re.I)
+    # --- Checkbox hack --- #
+    _CHECK_UNCHECKED = "\u2610"  # ☐
+    _CHECK_CHECKED = "\u2611"  # ☑
+    _CHECK_RX = re.compile(r"^\s*([\u2610\u2611])\s")  # ☐/☑ plus a space
+    _CHECKBOX_SCALE = 1.35
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, theme_manager: ThemeManager, *args, **kwargs):
         super().__init__(*args, **kwargs)
         tab_w = 4 * self.fontMetrics().horizontalAdvance(" ")
         self.setTabStopDistance(tab_w)
@@ -55,18 +64,173 @@ class Editor(QTextEdit):
 
         self.setAcceptRichText(True)
 
-        # Turn raw URLs into anchors
+        # If older docs have a baked-in color, normalize once:
+        self._retint_anchors_to_palette()
+
+        self._themes = theme_manager
+        self._apply_code_theme()  # set initial code colors
+        # Refresh on theme change
+        self._themes.themeChanged.connect(self._on_theme_changed)
+        self._themes.themeChanged.connect(
+            lambda _t: QTimer.singleShot(0, self._apply_code_theme)
+        )
+
         self._linkifying = False
         self.textChanged.connect(self._linkify_document)
         self.viewport().setMouseTracking(True)
 
+    # ---------------- Helpers ---------------- #
+
+    def _iter_frames(self, root=None):
+        """Depth-first traversal of all frames (including root if passed)."""
+        doc = self.document()
+        stack = [root or doc.rootFrame()]
+        while stack:
+            f = stack.pop()
+            yield f
+            it = f.begin()
+            while not it.atEnd():
+                cf = it.currentFrame()
+                if cf is not None:
+                    stack.append(cf)
+                it += 1
+
+    def _is_code_frame(self, frame, tolerant: bool = False) -> bool:
+        """
+        True if 'frame' is a code frame.
+        - tolerant=False: require our property marker
+        - tolerant=True: also accept legacy background or non-wrapping heuristic
+        """
+        ff = frame.frameFormat()
+        if ff.property(self._CODE_FRAME_PROP):
+            return True
+        if not tolerant:
+            return False
+
+        # Background colour check
+        bg = ff.background()
+        if bg.style() != Qt.NoBrush:
+            c = bg.color()
+            if c.isValid():
+                if (
+                    abs(c.red() - 245) <= 2
+                    and abs(c.green() - 245) <= 2
+                    and abs(c.blue() - 245) <= 2
+                ):
+                    return True
+                if (
+                    abs(c.red() - 43) <= 2
+                    and abs(c.green() - 43) <= 2
+                    and abs(c.blue() - 43) <= 2
+                ):
+                    return True
+
+        # Heuristic: mostly non-wrapping blocks
+        doc = self.document()
+        bc = QTextCursor(doc)
+        bc.setPosition(frame.firstPosition())
+        blocks = codeish = 0
+        while bc.position() < frame.lastPosition():
+            b = bc.block()
+            if not b.isValid():
+                break
+            blocks += 1
+            if b.blockFormat().nonBreakableLines():
+                codeish += 1
+            bc.setPosition(b.position() + b.length())
+        return blocks > 0 and (codeish / blocks) >= 0.6
+
+    def _nearest_code_frame(self, cursor, tolerant: bool = False):
+        """Walk up parents from the cursor and return the first code frame."""
+        f = cursor.currentFrame()
+        while f:
+            if self._is_code_frame(f, tolerant=tolerant):
+                return f
+            f = f.parentFrame()
+        return None
+
+    def _code_block_formats(self, fg: QColor | None = None):
+        """(QTextBlockFormat, QTextCharFormat) for code blocks."""
+        mono = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+
+        bf = QTextBlockFormat()
+        bf.setTopMargin(0)
+        bf.setBottomMargin(0)
+        bf.setLeftMargin(12)
+        bf.setRightMargin(12)
+        bf.setNonBreakableLines(True)
+
+        cf = QTextCharFormat()
+        cf.setFont(mono)
+        cf.setFontFixedPitch(True)
+        if fg is not None:
+            cf.setForeground(fg)
+        return bf, cf
+
+    def _new_code_frame_format(self, bg: QColor) -> QTextFrameFormat:
+        """Standard frame format for code blocks."""
+        ff = QTextFrameFormat()
+        ff.setBackground(bg)
+        ff.setPadding(6)
+        ff.setBorder(0)
+        ff.setLeftMargin(0)
+        ff.setRightMargin(0)
+        ff.setTopMargin(0)
+        ff.setBottomMargin(0)
+        ff.setProperty(self._CODE_FRAME_PROP, True)
+        return ff
+
+    def _retint_code_frame(self, frame, bg: QColor, fg: QColor | None):
+        """Apply background to frame and standard code formats to all blocks inside."""
+        ff = frame.frameFormat()
+        ff.setBackground(bg)
+        frame.setFrameFormat(ff)
+
+        bf, cf = self._code_block_formats(fg)
+        doc = self.document()
+        bc = QTextCursor(doc)
+        bc.setPosition(frame.firstPosition())
+        while bc.position() < frame.lastPosition():
+            bc.select(QTextCursor.BlockUnderCursor)
+            bc.mergeBlockFormat(bf)
+            bc.mergeBlockCharFormat(cf)
+            if not bc.movePosition(QTextCursor.NextBlock):
+                break
+
+    def _safe_block_insertion_cursor(self):
+        """
+        Return a cursor positioned for inserting an inline object (like an image):
+        - not inside a code frame (moves to after frame if necessary)
+        - at a fresh paragraph (inserts a block if mid-line)
+        Also updates the editor's current cursor to that position.
+        """
+        c = QTextCursor(self.textCursor())
+        frame = self._nearest_code_frame(c, tolerant=False)  # strict: our frames only
+        if frame:
+            out = QTextCursor(self.document())
+            out.setPosition(frame.lastPosition())
+            self.setTextCursor(out)
+            c = self.textCursor()
+        if c.positionInBlock() != 0:
+            c.insertBlock()
+        return c
+
+    def _scale_to_viewport(self, img: QImage, ratio: float = 0.92) -> QImage:
+        """If the image is wider than viewport*ratio, scale it down proportionally."""
+        if self.viewport():
+            max_w = int(self.viewport().width() * ratio)
+            if img.width() > max_w:
+                return img.scaledToWidth(max_w, Qt.SmoothTransformation)
+        return img
+
     def _approx(self, a: float, b: float, eps: float = 0.5) -> bool:
         return abs(float(a) - float(b)) <= eps
 
     def _is_heading_typing(self) -> bool:
         """Is the current *insertion* format using a heading size?"""
-        s = self.currentCharFormat().fontPointSize() or self.font().pointSizeF()
-        return any(self._approx(s, h) for h in self._HEADING_SIZES)
+        bf = self.textCursor().blockFormat()
+        if bf.headingLevel() > 0:
+            return True
 
     def _apply_normal_typing(self):
         """Switch the *insertion* format to Normal (default size, normal weight)."""
@@ -75,25 +239,35 @@ class Editor(QTextEdit):
         nf.setFontWeight(QFont.Weight.Normal)
         self.mergeCurrentCharFormat(nf)
 
-    def _find_code_frame(self, cursor=None):
-        """Return the nearest ancestor frame that's one of our code frames, else None."""
-        if cursor is None:
-            cursor = self.textCursor()
-        f = cursor.currentFrame()
-        while f:
-            if f.frameFormat().property(self._CODE_FRAME_PROP):
-                return f
-            f = f.parentFrame()
-        return None
+    def _code_theme_colors(self):
+        """Return (bg, fg) for code blocks based on the effective palette."""
+        pal = QApplication.instance().palette()
+        # simple luminance check on the window color
+        win = pal.color(QPalette.Window)
+        is_dark = win.value() < 128
+        if is_dark:
+            bg = QColor(43, 43, 43)  # dark code background
+            fg = pal.windowText().color()  # readable on dark
+        else:
+            bg = QColor(245, 245, 245)  # light code background
+            fg = pal.text().color()  # readable on light
+        return bg, fg
 
-    def _is_code_block(self, block) -> bool:
-        if not block.isValid():
-            return False
-        bf = block.blockFormat()
-        return bool(
-            bf.nonBreakableLines()
-            and bf.background().color().rgb() == self._CODE_BG.rgb()
-        )
+    def _apply_code_theme(self):
+        """Retint all code frames (even those reloaded from HTML) to match the current theme."""
+        bg, fg = self._code_theme_colors()
+        self._CODE_BG = bg  # used by future apply_code() calls
+
+        doc = self.document()
+        cur = QTextCursor(doc)
+        cur.beginEditBlock()
+        try:
+            for f in self._iter_frames(doc.rootFrame()):
+                if f is not doc.rootFrame() and self._is_code_frame(f, tolerant=True):
+                    self._retint_code_frame(f, bg, fg)
+        finally:
+            cur.endEditBlock()
+            self.viewport().update()
 
     def _trim_url_end(self, url: str) -> str:
         # strip common trailing punctuation not part of the URL
@@ -140,9 +314,9 @@ class Editor(QTextEdit):
                 fmt.setAnchor(True)
                 fmt.setAnchorHref(href)  # always refresh to the latest full URL
                 fmt.setFontUnderline(True)
-                fmt.setForeground(Qt.blue)
+                fmt.setForeground(self.palette().brush(QPalette.Link))
 
-                cur.mergeCharFormat(fmt)  # merge so we don’t clobber other styling
+                cur.mergeCharFormat(fmt)  # merge so we don't clobber other styling
 
             cur.endEditBlock()
         finally:
@@ -202,26 +376,12 @@ class Editor(QTextEdit):
             html = html.replace(f"src='{old}'", f"src='{data_url}'")
         return html
 
+    # ---------------- Image insertion & sizing (DRY’d) ---------------- #
+
     def _insert_qimage_at_cursor(self, img: QImage, autoscale=True):
-        c = self.textCursor()
-
-        # Don’t drop inside a code frame
-        frame = self._find_code_frame(c)
-        if frame:
-            out = QTextCursor(self.document())
-            out.setPosition(frame.lastPosition())
-            self.setTextCursor(out)
-            c = self.textCursor()
-
-        # Start a fresh paragraph if mid-line
-        if c.positionInBlock() != 0:
-            c.insertBlock()
-
-        if autoscale and self.viewport():
-            max_w = int(self.viewport().width() * 0.92)
-            if img.width() > max_w:
-                img = img.scaledToWidth(max_w, Qt.SmoothTransformation)
-
+        c = self._safe_block_insertion_cursor()
+        if autoscale:
+            img = self._scale_to_viewport(img)
         c.insertImage(img)
         c.insertBlock()  # one blank line after the image
 
@@ -331,6 +491,8 @@ class Editor(QTextEdit):
             return
         self._apply_image_size(tc, imgfmt, float(orig.width()), orig)
 
+    # ---------------- Context menu ---------------- #
+
     def contextMenuEvent(self, e):
         menu = self.createStandardContextMenu()
         tc, imgfmt, orig = self._image_info_at_cursor()
@@ -344,12 +506,14 @@ class Editor(QTextEdit):
             sub.addAction("Reset to original", self._reset_image_size)
         menu.exec(e.globalPos())
 
+    # ---------------- Clipboard / DnD ---------------- #
+
     def insertFromMimeData(self, source):
         # 1) Direct image from clipboard
         if source.hasImage():
             img = self._to_qimage(source.imageData())
             if img is not None:
-                self._insert_qimage_at_cursor(self, img, autoscale=True)
+                self._insert_qimage_at_cursor(img, autoscale=True)
                 return
 
         # 2) File URLs (drag/drop or paste)
@@ -391,7 +555,7 @@ class Editor(QTextEdit):
                     data = base64.b64decode(m.group(1))
                     img = QImage.fromData(data)
                     if not img.isNull():
-                        self._insert_qimage_at_cursor(self, img, autoscale=True)
+                        self._insert_qimage_at_cursor(img, autoscale=True)
                         return
                 except Exception:
                     pass  # fall through
@@ -405,19 +569,7 @@ class Editor(QTextEdit):
         Insert one or more images at the cursor. Large images can be auto-scaled
         to fit the viewport width while preserving aspect ratio.
         """
-        c = self.textCursor()
-
-        # Avoid dropping images inside a code frame
-        frame = self._find_code_frame(c)
-        if frame:
-            out = QTextCursor(self.document())
-            out.setPosition(frame.lastPosition())
-            self.setTextCursor(out)
-            c = self.textCursor()
-
-        # Ensure there's a paragraph break if we're mid-line
-        if c.positionInBlock() != 0:
-            c.insertBlock()
+        c = self._safe_block_insertion_cursor()
 
         for path in paths:
             reader = QImageReader(path)
@@ -425,14 +577,14 @@ class Editor(QTextEdit):
             if img.isNull():
                 continue
 
-            if autoscale and self.viewport():
-                max_w = int(self.viewport().width() * 0.92)  # ~92% of editor width
-                if img.width() > max_w:
-                    img = img.scaledToWidth(max_w, Qt.SmoothTransformation)
+            if autoscale:
+                img = self._scale_to_viewport(img)
 
             c.insertImage(img)
             c.insertBlock()  # put each image on its own line
 
+    # ---------------- Mouse & key handling ---------------- #
+
     def mouseReleaseEvent(self, e):
         if e.button() == Qt.LeftButton and (e.modifiers() & Qt.ControlModifier):
             href = self.anchorAt(e.pos())
@@ -449,11 +601,52 @@ class Editor(QTextEdit):
             self.viewport().setCursor(Qt.IBeamCursor)
         super().mouseMoveEvent(e)
 
+    def mousePressEvent(self, e):
+        if e.button() == Qt.LeftButton and not (e.modifiers() & Qt.ControlModifier):
+            cur = self.cursorForPosition(e.pos())
+            b = cur.block()
+            state, pref = self._checkbox_info_for_block(b)
+            if state is not None:
+                col = cur.position() - b.position()
+                if col <= max(1, pref):  # clicked on ☐/☑ (and the following space)
+                    self._set_block_checkbox_state(b, not state)
+                    return
+        return super().mousePressEvent(e)
+
     def keyPressEvent(self, e):
         key = e.key()
 
-        # Pre-insert: stop link/format bleed for “word boundary” keys
         if key in (Qt.Key_Space, Qt.Key_Tab):
+            c = self.textCursor()
+            b = c.block()
+            pos_in_block = c.position() - b.position()
+
+            if (
+                pos_in_block >= 4
+                and b.text().startswith("TODO")
+                and b.text()[:pos_in_block] == "TODO"
+                and self._checkbox_info_for_block(b)[0] is None
+            ):
+                tcur = QTextCursor(self.document())
+                tcur.setPosition(b.position())  # start of block
+                tcur.setPosition(
+                    b.position() + 4, QTextCursor.KeepAnchor
+                )  # select "TODO"
+                tcur.beginEditBlock()
+                tcur.removeSelectedText()
+                tcur.insertText(self._CHECK_UNCHECKED + " ")  # insert "☐ "
+                tcur.endEditBlock()
+
+                # visuals: size bump
+                if hasattr(self, "_style_checkbox_glyph"):
+                    self._style_checkbox_glyph(b)
+
+                # caret after the inserted prefix; swallow the key (we already added a space)
+                c.setPosition(b.position() + 2)
+                self.setTextCursor(c)
+                return
+
+            # not a TODO-at-start case
             self._break_anchor_for_next_char()
             return super().keyPressEvent(e)
 
@@ -462,7 +655,7 @@ class Editor(QTextEdit):
 
             # If we're on an empty line inside a code frame, consume Enter and jump out
             if c.block().length() == 1:
-                frame = self._find_code_frame(c)
+                frame = self._nearest_code_frame(c, tolerant=False)
                 if frame:
                     out = QTextCursor(self.document())
                     out.setPosition(frame.lastPosition())  # after the frame's contents
@@ -470,6 +663,26 @@ class Editor(QTextEdit):
                     super().insertPlainText("\n")  # start a normal paragraph
                     return
 
+            # --- CHECKBOX handling: continue on Enter; "escape" on second Enter ---
+            b = c.block()
+            state, pref = self._checkbox_info_for_block(b)
+            if state is not None and not c.hasSelection():
+                text_after = b.text()[pref:].strip()
+                if c.atBlockEnd() and text_after == "":
+                    # Empty checkbox item -> remove the prefix and insert a plain new line
+                    cur = QTextCursor(self.document())
+                    cur.setPosition(b.position())
+                    cur.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, pref)
+                    cur.removeSelectedText()
+                    return super().keyPressEvent(e)
+                else:
+                    # Normal continuation: new checkbox on the next line
+                    super().keyPressEvent(e)  # make the new block
+                    super().insertPlainText(self._CHECK_UNCHECKED + " ")
+                    if hasattr(self, "_style_checkbox_glyph"):
+                        self._style_checkbox_glyph(self.textCursor().block())
+                    return
+
             # Follow-on style: if we typed a heading and press Enter at end of block,
             # new paragraph should revert to Normal.
             if not c.hasSelection() and c.atBlockEnd() and self._is_heading_typing():
@@ -480,11 +693,6 @@ class Editor(QTextEdit):
         # otherwise default handling
         return super().keyPressEvent(e)
 
-    def _clear_insertion_char_format(self):
-        """Reset inline typing format (keeps lists, alignment, margins, etc.)."""
-        nf = QTextCharFormat()
-        self.setCurrentCharFormat(nf)
-
     def _break_anchor_for_next_char(self):
         """
         Ensure the *next* typed character is not part of a hyperlink.
@@ -496,12 +704,21 @@ class Editor(QTextEdit):
         cur_fmt = self.textCursor().charFormat()
 
         # Do nothing unless either side indicates we're in/propagating an anchor
-        if not (ins_fmt.isAnchor() or cur_fmt.isAnchor()):
+        if not (
+            ins_fmt.isAnchor()
+            or cur_fmt.isAnchor()
+            or ins_fmt.fontUnderline()
+            or ins_fmt.foreground().style() != Qt.NoBrush
+        ):
             return
 
         nf = QTextCharFormat(ins_fmt)
+        # stop the link itself
         nf.setAnchor(False)
         nf.setAnchorHref("")
+        # also stop the link *styling*
+        nf.setFontUnderline(False)
+        nf.clearForeground()
 
         self.setCurrentCharFormat(nf)
 
@@ -514,6 +731,117 @@ class Editor(QTextEdit):
             cursor.mergeCharFormat(fmt)
         self.mergeCurrentCharFormat(fmt)
 
+    # ====== Checkbox core ======
+    def _base_point_size_for_block(self, block) -> float:
+        # Try the block's char format, then editor font
+        sz = block.charFormat().fontPointSize()
+        if sz <= 0:
+            sz = self.fontPointSize()
+        if sz <= 0:
+            sz = self.font().pointSizeF() or 12.0
+        return float(sz)
+
+    def _style_checkbox_glyph(self, block):
+        """Apply larger size (and optional symbol font) to the single ☐/☑ char."""
+        state, _ = self._checkbox_info_for_block(block)
+        if state is None:
+            return
+        doc = self.document()
+        c = QTextCursor(doc)
+        c.setPosition(block.position())
+        c.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, 1)  # select ☐/☑ only
+
+        base = self._base_point_size_for_block(block)
+        fmt = QTextCharFormat()
+        fmt.setFontPointSize(base * self._CHECKBOX_SCALE)
+        # keep the glyph centered on the text baseline
+        fmt.setVerticalAlignment(QTextCharFormat.AlignMiddle)
+
+        c.mergeCharFormat(fmt)
+
+    def _checkbox_info_for_block(self, block):
+        """Return (state, prefix_len): state in {None, False, True}, prefix_len in chars."""
+        text = block.text()
+        m = self._CHECK_RX.match(text)
+        if not m:
+            return None, 0
+        ch = m.group(1)
+        state = True if ch == self._CHECK_CHECKED else False
+        return state, m.end()
+
+    def _set_block_checkbox_present(self, block, present: bool):
+        state, pref = self._checkbox_info_for_block(block)
+        doc = self.document()
+        c = QTextCursor(doc)
+        c.setPosition(block.position())
+        c.beginEditBlock()
+        try:
+            if present and state is None:
+                c.insertText(self._CHECK_UNCHECKED + " ")
+                state = False
+                self._style_checkbox_glyph(block)
+            else:
+                if state is not None:
+                    c.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, pref)
+                    c.removeSelectedText()
+                    state = None
+        finally:
+            c.endEditBlock()
+
+        return state
+
+    def _set_block_checkbox_state(self, block, checked: bool):
+        """Switch ☐/☑ at the start of the block."""
+        state, pref = self._checkbox_info_for_block(block)
+        if state is None:
+            return
+        doc = self.document()
+        c = QTextCursor(doc)
+        c.setPosition(block.position())
+        c.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, 1)  # just the symbol
+        c.beginEditBlock()
+        try:
+            c.removeSelectedText()
+            c.insertText(self._CHECK_CHECKED if checked else self._CHECK_UNCHECKED)
+            self._style_checkbox_glyph(block)
+        finally:
+            c.endEditBlock()
+
+    # Public API used by toolbar
+    def toggle_checkboxes(self):
+        """
+        Toggle checkbox prefix on/off for the current block(s).
+        If all targeted blocks already have a checkbox, remove them; otherwise add.
+        """
+        c = self.textCursor()
+        doc = self.document()
+
+        if c.hasSelection():
+            start = doc.findBlock(c.selectionStart())
+            end = doc.findBlock(c.selectionEnd() - 1)
+        else:
+            start = end = c.block()
+
+        # Decide intent: add or remove?
+        b = start
+        all_have = True
+        while True:
+            state, _ = self._checkbox_info_for_block(b)
+            if state is None:
+                all_have = False
+                break
+            if b == end:
+                break
+            b = b.next()
+
+        # Apply
+        b = start
+        while True:
+            self._set_block_checkbox_present(b, present=not all_have)
+            if b == end:
+                break
+            b = b.next()
+
     @Slot()
     def apply_weight(self):
         cur = self.currentCharFormat()
@@ -553,46 +881,16 @@ class Editor(QTextEdit):
         if not c.hasSelection():
             c.select(QTextCursor.BlockUnderCursor)
 
-        # Wrap the selection in a single frame (no per-block padding/margins).
-        ff = QTextFrameFormat()
-        ff.setBackground(self._CODE_BG)
-        ff.setPadding(6)  # visual padding for the WHOLE block
-        ff.setBorder(0)
-        ff.setLeftMargin(0)
-        ff.setRightMargin(0)
-        ff.setTopMargin(0)
-        ff.setBottomMargin(0)
-        ff.setProperty(self._CODE_FRAME_PROP, True)
-
-        mono = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+        ff = self._new_code_frame_format(self._CODE_BG)
 
         c.beginEditBlock()
         try:
             c.insertFrame(ff)  # with a selection, this wraps the selection
 
-            # Format all blocks inside the new frame: zero vertical margins, mono font, no wrapping
-            frame = self._find_code_frame(c)
-            bc = QTextCursor(self.document())
-            bc.setPosition(frame.firstPosition())
-
-            while bc.position() < frame.lastPosition():
-                bc.select(QTextCursor.BlockUnderCursor)
-
-                bf = QTextBlockFormat()
-                bf.setTopMargin(0)
-                bf.setBottomMargin(0)
-                bf.setLeftMargin(12)
-                bf.setRightMargin(12)
-                bf.setNonBreakableLines(True)
-
-                cf = QTextCharFormat()
-                cf.setFont(mono)
-                cf.setFontFixedPitch(True)
-
-                bc.mergeBlockFormat(bf)
-                bc.mergeBlockCharFormat(cf)
-
-                bc.setPosition(bc.block().position() + bc.block().length())
+            # Format all blocks inside the new frame (keep fg=None on creation)
+            frame = self._nearest_code_frame(c, tolerant=False)
+            if frame:
+                self._retint_code_frame(frame, self._CODE_BG, fg=None)
         finally:
             c.endEditBlock()
 
@@ -602,20 +900,43 @@ class Editor(QTextEdit):
         Set heading point size for typing. If there's a selection, also apply bold
         to that selection (for H1..H3). "Normal" clears bold on the selection.
         """
-        base_size = size if size else self.font().pointSizeF()
+        # Map toolbar's sizes to heading levels
+        level = 1 if size >= 24 else 2 if size >= 18 else 3 if size >= 14 else 0
+
         c = self.textCursor()
 
-        # Update the typing (insertion) format to be size only, but don't represent
-        # it as if the Bold style has been toggled on
+        # On-screen look
         ins = QTextCharFormat()
-        ins.setFontPointSize(base_size)
+        if size:
+            ins.setFontPointSize(float(size))
+            ins.setFontWeight(QFont.Weight.Bold)
+        else:
+            ins.setFontPointSize(self.font().pointSizeF())
+            ins.setFontWeight(QFont.Weight.Normal)
         self.mergeCurrentCharFormat(ins)
 
-        # If user selected text, style that text visually as a heading
+        # Apply heading level to affected block(s)
+        def set_level_for_block(cur):
+            bf = cur.blockFormat()
+            if hasattr(bf, "setHeadingLevel"):
+                bf.setHeadingLevel(level)  # 0 clears heading
+                cur.mergeBlockFormat(bf)
+
         if c.hasSelection():
-            sel = QTextCharFormat(ins)
-            sel.setFontWeight(QFont.Weight.Bold if size else QFont.Weight.Normal)
-            c.mergeCharFormat(sel)
+            start, end = c.selectionStart(), c.selectionEnd()
+            bc = QTextCursor(self.document())
+            bc.setPosition(start)
+            while True:
+                set_level_for_block(bc)
+                if bc.position() >= end:
+                    break
+                bc.movePosition(QTextCursor.EndOfBlock)
+                if bc.position() >= end:
+                    break
+                bc.movePosition(QTextCursor.NextBlock)
+        else:
+            bc = QTextCursor(c)
+            set_level_for_block(bc)
 
     def toggle_bullets(self):
         c = self.textCursor()
@@ -636,3 +957,53 @@ class Editor(QTextEdit):
         fmt = QTextListFormat()
         fmt.setStyle(QTextListFormat.Style.ListDecimal)
         c.createList(fmt)
+
+    @Slot(Theme)
+    def _on_theme_changed(self, _theme: Theme):
+        # Defer one event-loop tick so widgets have the new palette
+        QTimer.singleShot(0, self._retint_anchors_to_palette)
+        QTimer.singleShot(0, self._apply_code_theme)
+
+    @Slot()
+    def _retint_anchors_to_palette(self, *_):
+        # Always read from the *application* palette to avoid stale widget palette
+        app = QApplication.instance()
+        link_brush = app.palette().brush(QPalette.Link)
+        doc = self.document()
+        cur = QTextCursor(doc)
+        cur.beginEditBlock()
+        block = doc.firstBlock()
+        while block.isValid():
+            it = block.begin()
+            while not it.atEnd():
+                frag = it.fragment()
+                if frag.isValid():
+                    fmt = frag.charFormat()
+                    if fmt.isAnchor():
+                        new_fmt = QTextCharFormat(fmt)
+                        new_fmt.setForeground(link_brush)  # force palette link color
+                        start = frag.position()
+                        cur.setPosition(start)
+                        cur.movePosition(
+                            QTextCursor.NextCharacter,
+                            QTextCursor.KeepAnchor,
+                            frag.length(),
+                        )  # select exactly this fragment
+                        cur.setCharFormat(new_fmt)
+                it += 1
+            block = block.next()
+        cur.endEditBlock()
+        self.viewport().update()
+
+    def setHtml(self, html: str) -> None:
+        super().setHtml(html)
+
+        doc = self.document()
+        block = doc.firstBlock()
+        while block.isValid():
+            self._style_checkbox_glyph(block)  # Apply checkbox styling to each block
+            block = block.next()
+
+        # Ensure anchors adopt the palette color on startup
+        self._retint_anchors_to_palette()
+        self._apply_code_theme()
