@@ -18,6 +18,7 @@ from PySide6.QtWidgets import (
 
 def _html_to_text(s: str) -> str:
     """Lightweight HTML→text for diff (keeps paragraphs/line breaks)."""
+    IMG_RE = re.compile(r"(?is)<img\b[^>]*>")
     STYLE_SCRIPT_RE = re.compile(r"(?is)<(script|style)[^>]*>.*?</\1>")
     COMMENT_RE = re.compile(r"<!--.*?-->", re.S)
     BR_RE = re.compile(r"(?i)<br\s*/?>")
@@ -25,6 +26,7 @@ def _html_to_text(s: str) -> str:
     TAG_RE = re.compile(r"<[^>]+>")
     MULTINL_RE = re.compile(r"\n{3,}")
 
+    s = IMG_RE.sub("[ Image changed - see Preview pane ]", s)
     s = STYLE_SCRIPT_RE.sub("", s)
     s = COMMENT_RE.sub("", s)
     s = BR_RE.sub("\n", s)
@@ -154,7 +156,7 @@ class HistoryDialog(QDialog):
         # Diff vs current (textual diff)
         cur = self._db.get_version(version_id=self._current_id)
         self.diff.setHtml(_colored_unified_diff_html(cur["content"], sel["content"]))
-        # Enable revert only if selecting a non-current
+        # Enable revert only if selecting a non-current version
         self.btn_revert.setEnabled(sel_id != self._current_id)
 
     @Slot()
@@ -165,12 +167,10 @@ class HistoryDialog(QDialog):
         sel_id = item.data(Qt.UserRole)
         if sel_id == self._current_id:
             return
-        sel = self._db.get_version(version_id=sel_id)
-        vno = sel["version_no"]
-        # Flip head pointer
+        # Flip head pointer to the older version
         try:
             self._db.revert_to_version(self._date, version_id=sel_id)
         except Exception as e:
             QMessageBox.critical(self, "Revert failed", str(e))
             return
-        self.accept()  # let the caller refresh the editor
+        self.accept()
