@@ -3,6 +3,7 @@ from __future__ import annotations
 import datetime
 import os
 import sys
+import re
 
 from pathlib import Path
 from PySide6.QtCore import (
@@ -17,21 +18,25 @@ from PySide6.QtCore import (
 )
 from PySide6.QtGui import (
     QAction,
+    QBrush,
+    QColor,
     QCursor,
     QDesktopServices,
     QFont,
     QGuiApplication,
+    QKeySequence,
+    QPalette,
     QTextCharFormat,
+    QTextCursor,
     QTextListFormat,
 )
 from PySide6.QtWidgets import (
+    QApplication,
     QCalendarWidget,
     QDialog,
     QFileDialog,
-    QLabel,
     QMainWindow,
     QMessageBox,
-    QPushButton,
     QSizePolicy,
     QSplitter,
     QVBoxLayout,
@@ -40,76 +45,26 @@ from PySide6.QtWidgets import (
 
 from .db import DBManager
 from .editor import Editor
+from .find_bar import FindBar
 from .history_dialog import HistoryDialog
 from .key_prompt import KeyPrompt
+from .lock_overlay import LockOverlay
 from .save_dialog import SaveDialog
 from .search import Search
 from .settings import APP_ORG, APP_NAME, load_db_config, save_db_config
 from .settings_dialog import SettingsDialog
 from .toolbar import ToolBar
-
-
-class _LockOverlay(QWidget):
-    def __init__(self, parent: QWidget, on_unlock: callable):
-        super().__init__(parent)
-        self.setObjectName("LockOverlay")
-        self.setAttribute(Qt.WA_StyledBackground, True)
-        self.setFocusPolicy(Qt.StrongFocus)
-        self.setGeometry(parent.rect())
-
-        self.setStyleSheet(
-            """
-#LockOverlay { background-color: #ccc; }
-#LockOverlay QLabel { color: #fff; font-size: 18px; }
-#LockOverlay QPushButton {
-    background-color: #f2f2f2;
-    color: #000;
-    padding: 6px 14px;
-    border: 1px solid #808080;
-    border-radius: 6px;
-    font-size: 14px;
-}
-#LockOverlay QPushButton:hover   { background-color: #ffffff; }
-#LockOverlay QPushButton:pressed { background-color: #e6e6e6; }
-"""
-        )
-
-        lay = QVBoxLayout(self)
-        lay.addStretch(1)
-
-        msg = QLabel("Locked due to inactivity")
-        msg.setAlignment(Qt.AlignCenter)
-
-        self._btn = QPushButton("Unlock")
-        self._btn.setFixedWidth(200)
-        self._btn.setCursor(Qt.PointingHandCursor)
-        self._btn.setAutoDefault(True)
-        self._btn.setDefault(True)
-        self._btn.clicked.connect(on_unlock)
-
-        lay.addWidget(msg, 0, Qt.AlignCenter)
-        lay.addWidget(self._btn, 0, Qt.AlignCenter)
-        lay.addStretch(1)
-
-        self.hide()  # start hidden
-
-    # keep overlay sized with its parent
-    def eventFilter(self, obj, event):
-        if obj is self.parent() and event.type() in (QEvent.Resize, QEvent.Show):
-            self.setGeometry(obj.rect())
-        return False
-
-    def showEvent(self, e):
-        super().showEvent(e)
-        self._btn.setFocus()
+from .theme import Theme, ThemeManager
 
 
 class MainWindow(QMainWindow):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, themes: ThemeManager, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.setWindowTitle(APP_NAME)
         self.setMinimumSize(1000, 650)
 
+        self.themes = themes  # Store the themes manager
+
         self.cfg = load_db_config()
         if not os.path.exists(self.cfg.path):
             # Fresh database/first time use, so guide the user re: setting a key
@@ -132,19 +87,19 @@ class MainWindow(QMainWindow):
 
         self.search = Search(self.db)
         self.search.openDateRequested.connect(self._load_selected_date)
+        self.search.resultDatesChanged.connect(self._on_search_dates_changed)
 
         # Lock the calendar to the left panel at the top to stop it stretching
         # when the main window is resized.
         left_panel = QWidget()
         left_layout = QVBoxLayout(left_panel)
         left_layout.setContentsMargins(8, 8, 8, 8)
-        left_layout.addWidget(self.calendar, alignment=Qt.AlignTop)
-        left_layout.addWidget(self.search, alignment=Qt.AlignBottom)
-        left_layout.addStretch(1)
+        left_layout.addWidget(self.calendar)
+        left_layout.addWidget(self.search)
         left_panel.setFixedWidth(self.calendar.sizeHint().width() + 16)
 
         # This is the note-taking editor
-        self.editor = Editor()
+        self.editor = Editor(self.themes)
 
         # Toolbar for controlling styling
         self.toolBar = ToolBar()
@@ -158,6 +113,7 @@ class MainWindow(QMainWindow):
         self.toolBar.headingRequested.connect(self.editor.apply_heading)
         self.toolBar.bulletsRequested.connect(self.editor.toggle_bullets)
         self.toolBar.numbersRequested.connect(self.editor.toggle_numbers)
+        self.toolBar.checkboxesRequested.connect(self.editor.toggle_checkboxes)
         self.toolBar.alignRequested.connect(self.editor.setAlignment)
         self.toolBar.historyRequested.connect(self._open_history)
         self.toolBar.insertImageRequested.connect(self._on_insert_image)
@@ -168,7 +124,7 @@ class MainWindow(QMainWindow):
         split = QSplitter()
         split.addWidget(left_panel)
         split.addWidget(self.editor)
-        split.setStretchFactor(1, 1)  # editor grows
+        split.setStretchFactor(1, 1)
 
         container = QWidget()
         lay = QVBoxLayout(container)
@@ -183,7 +139,7 @@ class MainWindow(QMainWindow):
         self._idle_timer.start()
 
         # full-window overlay that sits on top of the central widget
-        self._lock_overlay = _LockOverlay(self.centralWidget(), self._on_unlock_clicked)
+        self._lock_overlay = LockOverlay(self.centralWidget(), self._on_unlock_clicked)
         self.centralWidget().installEventFilter(self._lock_overlay)
 
         self._locked = False
@@ -193,8 +149,23 @@ class MainWindow(QMainWindow):
 
         QApplication.instance().installEventFilter(self)
 
+        # Focus on the editor
+        self.setFocusPolicy(Qt.StrongFocus)
+        self.editor.setFocusPolicy(Qt.StrongFocus)
+        self.toolBar.setFocusPolicy(Qt.NoFocus)
+        for w in self.toolBar.findChildren(QWidget):
+            w.setFocusPolicy(Qt.NoFocus)
+        QGuiApplication.instance().applicationStateChanged.connect(
+            self._on_app_state_changed
+        )
+
         # Status bar for feedback
         self.statusBar().showMessage("Ready", 800)
+        # Add findBar and add it to the statusBar
+        self.findBar = FindBar(self.editor, shortcut_parent=self, parent=self)
+        self.statusBar().addPermanentWidget(self.findBar)
+        # When the findBar closes, put the caret back in the editor
+        self.findBar.closed.connect(self._focus_editor_now)
 
         # Menu bar (File)
         mb = self.menuBar()
@@ -249,6 +220,24 @@ class MainWindow(QMainWindow):
         nav_menu.addAction(act_today)
         self.addAction(act_today)
 
+        act_find = QAction("Find on page", self)
+        act_find.setShortcut(QKeySequence.Find)
+        act_find.triggered.connect(self.findBar.show_bar)
+        nav_menu.addAction(act_find)
+        self.addAction(act_find)
+
+        act_find_next = QAction("Find Next", self)
+        act_find_next.setShortcut(QKeySequence.FindNext)
+        act_find_next.triggered.connect(self.findBar.find_next)
+        nav_menu.addAction(act_find_next)
+        self.addAction(act_find_next)
+
+        act_find_prev = QAction("Find Previous", self)
+        act_find_prev.setShortcut(QKeySequence.FindPrevious)
+        act_find_prev.triggered.connect(self.findBar.find_prev)
+        nav_menu.addAction(act_find_prev)
+        self.addAction(act_find_prev)
+
         # Help menu with drop-down
         help_menu = mb.addMenu("&Help")
         act_docs = QAction("Documentation", self)
@@ -272,13 +261,24 @@ class MainWindow(QMainWindow):
         self.editor.textChanged.connect(self._on_text_changed)
 
         # First load + mark dates in calendar with content
-        self._load_selected_date()
+        if not self._load_yesterday_todos():
+            self._load_selected_date()
         self._refresh_calendar_marks()
 
         # Restore window position from settings
         self.settings = QSettings(APP_ORG, APP_NAME)
         self._restore_window_position()
 
+        self._apply_link_css()  # Apply link color on startup
+        # re-apply all runtime color tweaks when theme changes
+        self.themes.themeChanged.connect(lambda _t: self._retheme_overrides())
+        self.themes.themeChanged.connect(self._apply_calendar_theme)
+        self._apply_calendar_text_colors()
+        self._apply_calendar_theme(self.themes.current())
+
+        # apply once on startup so links / calendar colors are set immediately
+        self._retheme_overrides()
+
     def _try_connect(self) -> bool:
         """
         Try to connect to the database.
@@ -313,22 +313,134 @@ class MainWindow(QMainWindow):
             if self._try_connect():
                 return True
 
+    def _retheme_overrides(self):
+        if hasattr(self, "_lock_overlay"):
+            self._lock_overlay._apply_overlay_style()
+        self._apply_calendar_text_colors()
+        self._apply_link_css()
+        self._apply_search_highlights(getattr(self, "_search_highlighted_dates", set()))
+        self.calendar.update()
+        self.editor.viewport().update()
+
+    def _apply_link_css(self):
+        if self.themes and self.themes.current() == Theme.DARK:
+            anchor = Theme.ORANGE_ANCHOR.value
+            visited = Theme.ORANGE_ANCHOR_VISITED.value
+            css = f"""
+                a {{ color: {anchor}; text-decoration: underline; }}
+                a:visited {{ color: {visited}; }}
+            """
+        else:
+            css = ""  # Default to no custom styling for links (system or light theme)
+
+        try:
+            self.editor.document().setDefaultStyleSheet(css)
+        except Exception:
+            pass
+
+        try:
+            self.search.document().setDefaultStyleSheet(css)
+        except Exception:
+            pass
+
+    def _apply_calendar_theme(self, theme: Theme):
+        """Use orange accents on the calendar in dark mode only."""
+        app_pal = QApplication.instance().palette()
+
+        if theme == Theme.DARK:
+            highlight = QColor(Theme.ORANGE_ANCHOR.value)
+            black = QColor(0, 0, 0)
+
+            highlight_css = Theme.ORANGE_ANCHOR.value
+
+            # Per-widget palette: selection color inside the date grid
+            pal = self.calendar.palette()
+            pal.setColor(QPalette.Highlight, highlight)
+            pal.setColor(QPalette.HighlightedText, black)
+            self.calendar.setPalette(pal)
+
+            # Stylesheet: nav bar + selected-day background
+            self.calendar.setStyleSheet(
+                f"""
+                QWidget#qt_calendar_navigationbar {{ background-color: {highlight_css}; }}
+                QCalendarWidget QToolButton {{ color: black; }}
+                QCalendarWidget QToolButton:hover {{ background-color: rgba(255,165,0,0.20); }}
+                /* Selected day color in the table view */
+                QCalendarWidget QTableView:enabled {{
+                    selection-background-color: {highlight_css};
+                    selection-color: black;
+                }}
+                /* Optional: keep weekday header readable */
+                QCalendarWidget QTableView QHeaderView::section {{
+                    background: transparent;
+                    color: palette(windowText);
+                }}
+            """
+            )
+        else:
+            # Back to app defaults in light/system
+            self.calendar.setPalette(app_pal)
+            self.calendar.setStyleSheet("")
+
+        self._apply_calendar_text_colors()
+        self.calendar.update()
+
+    def _apply_calendar_text_colors(self):
+        pal = self.palette()
+        txt = pal.windowText().color()
+        fmt = QTextCharFormat()
+        fmt.setForeground(txt)
+        # Use normal text color for weekends
+        self.calendar.setWeekdayTextFormat(Qt.Saturday, fmt)
+        self.calendar.setWeekdayTextFormat(Qt.Sunday, fmt)
+
+    def _on_search_dates_changed(self, date_strs: list[str]):
+        dates = set()
+        for ds in date_strs or []:
+            qd = QDate.fromString(ds, "yyyy-MM-dd")
+            if qd.isValid():
+                dates.add(qd)
+        self._apply_search_highlights(dates)
+
+    def _apply_search_highlights(self, dates: set):
+        pal = self.palette()
+        base = pal.base().color()
+        hi = pal.highlight().color()
+        # Blend highlight with base so it looks soft in both modes
+        blend = QColor(
+            (2 * hi.red() + base.red()) // 3,
+            (2 * hi.green() + base.green()) // 3,
+            (2 * hi.blue() + base.blue()) // 3,
+        )
+        yellow = QBrush(blend)
+        old = getattr(self, "_search_highlighted_dates", set())
+
+        for d in old - dates:  # clear removed
+            fmt = self.calendar.dateTextFormat(d)
+            fmt.setBackground(Qt.transparent)
+            self.calendar.setDateTextFormat(d, fmt)
+
+        for d in dates:  # apply new/current
+            fmt = self.calendar.dateTextFormat(d)
+            fmt.setBackground(yellow)
+            self.calendar.setDateTextFormat(d, fmt)
+
+        self._search_highlighted_dates = dates
+
     def _refresh_calendar_marks(self):
-        """
-        Sets a bold marker on the day to indicate that text exists
-        for that day.
-        """
-        fmt_bold = QTextCharFormat()
-        fmt_bold.setFontWeight(QFont.Weight.Bold)
-        # Clear previous marks
+        """Make days with entries bold, but keep any search highlight backgrounds."""
         for d in getattr(self, "_marked_dates", set()):
-            self.calendar.setDateTextFormat(d, QTextCharFormat())
+            fmt = self.calendar.dateTextFormat(d)
+            fmt.setFontWeight(QFont.Weight.Normal)  # remove bold only
+            self.calendar.setDateTextFormat(d, fmt)
         self._marked_dates = set()
         try:
             for date_iso in self.db.dates_with_content():
                 qd = QDate.fromString(date_iso, "yyyy-MM-dd")
                 if qd.isValid():
-                    self.calendar.setDateTextFormat(qd, fmt_bold)
+                    fmt = self.calendar.dateTextFormat(qd)
+                    fmt.setFontWeight(QFont.Weight.Bold)  # add bold only
+                    self.calendar.setDateTextFormat(qd, fmt)
                     self._marked_dates.add(qd)
         except Exception:
             pass
@@ -341,10 +453,10 @@ class MainWindow(QMainWindow):
         bf = c.blockFormat()
 
         # Block signals so setChecked() doesn't re-trigger actions
-        blocker1 = QSignalBlocker(self.toolBar.actBold)
-        blocker2 = QSignalBlocker(self.toolBar.actItalic)
-        blocker3 = QSignalBlocker(self.toolBar.actUnderline)
-        blocker4 = QSignalBlocker(self.toolBar.actStrike)
+        QSignalBlocker(self.toolBar.actBold)
+        QSignalBlocker(self.toolBar.actItalic)
+        QSignalBlocker(self.toolBar.actUnderline)
+        QSignalBlocker(self.toolBar.actStrike)
 
         self.toolBar.actBold.setChecked(fmt.fontWeight() == QFont.Weight.Bold)
         self.toolBar.actItalic.setChecked(fmt.fontItalic())
@@ -361,10 +473,10 @@ class MainWindow(QMainWindow):
         bH2 = _approx(cur_size, 18)
         bH3 = _approx(cur_size, 14)
 
-        b1 = QSignalBlocker(self.toolBar.actH1)
-        b2 = QSignalBlocker(self.toolBar.actH2)
-        b3 = QSignalBlocker(self.toolBar.actH3)
-        bN = QSignalBlocker(self.toolBar.actNormal)
+        QSignalBlocker(self.toolBar.actH1)
+        QSignalBlocker(self.toolBar.actH2)
+        QSignalBlocker(self.toolBar.actH3)
+        QSignalBlocker(self.toolBar.actNormal)
 
         self.toolBar.actH1.setChecked(bH1)
         self.toolBar.actH2.setChecked(bH2)
@@ -393,17 +505,29 @@ class MainWindow(QMainWindow):
         d = self.calendar.selectedDate()
         return f"{d.year():04d}-{d.month():02d}-{d.day():02d}"
 
-    def _load_selected_date(self, date_iso=False):
+    def _load_selected_date(self, date_iso=False, extra_data=False):
         if not date_iso:
             date_iso = self._current_date_iso()
         try:
             text = self.db.get_entry(date_iso)
+            if extra_data:
+                # Wrap extra_data in a <p> tag for HTML rendering
+                extra_data_html = f"<p>{extra_data}</p>"
+
+                # Inject the extra_data before the closing </body></html>
+                modified = re.sub(r"(<\/body><\/html>)", extra_data_html + r"\1", text)
+                text = modified
+                # Force a save now so we don't lose it.
+                self._set_editor_html_preserve_view(text)
+                self._dirty = True
+                self._save_date(date_iso, True)
+
         except Exception as e:
             QMessageBox.critical(self, "Read Error", str(e))
             return
-        self.editor.blockSignals(True)
-        self.editor.setHtml(text)
-        self.editor.blockSignals(False)
+
+        self._set_editor_html_preserve_view(text)
+
         self._dirty = False
         # track which date the editor currently represents
         self._active_date_iso = date_iso
@@ -412,7 +536,7 @@ class MainWindow(QMainWindow):
 
     def _on_text_changed(self):
         self._dirty = True
-        self._save_timer.start(10000)  # autosave after idle
+        self._save_timer.start(5000)  # autosave after idle
 
     def _adjust_day(self, delta: int):
         """Move selection by delta days (negative for previous)."""
@@ -424,6 +548,56 @@ class MainWindow(QMainWindow):
         today = QDate.currentDate()
         self.calendar.setSelectedDate(today)
 
+    def _load_yesterday_todos(self):
+        try:
+            if not self.cfg.move_todos:
+                return
+            yesterday_str = QDate.currentDate().addDays(-1).toString("yyyy-MM-dd")
+            text = self.db.get_entry(yesterday_str)
+            unchecked_items = []
+
+            # Regex to match the unchecked checkboxes and their associated text
+            checkbox_pattern = re.compile(
+                r"<span[^>]*>(☐)</span>\s*(.*?)</p>", re.DOTALL
+            )
+
+            # Find unchecked items and store them
+            for match in checkbox_pattern.finditer(text):
+                checkbox = match.group(1)  # Either ☐ or ☑
+                item_text = match.group(2).strip()  # The text after the checkbox
+                if checkbox == "☐":  # If it's an unchecked checkbox (☐)
+                    unchecked_items.append("☐ " + item_text)  # Store the unchecked item
+
+            # Remove the unchecked items from yesterday's HTML content
+            if unchecked_items:
+                # This regex will find the entire checkbox line and remove it from the HTML content
+                uncheckbox_pattern = re.compile(
+                    r"<span[^>]*>☐</span>\s*(.*?)</p>", re.DOTALL
+                )
+                modified_text = re.sub(
+                    uncheckbox_pattern, "", text
+                )  # Remove the checkbox lines
+
+                # Save the modified HTML back to the database
+                self.db.save_new_version(
+                    yesterday_str,
+                    modified_text,
+                    "Unchecked checkbox items moved to next day",
+                )
+
+                # Join unchecked items into a formatted string
+                unchecked_str = "\n".join(
+                    [f"<p>{item}</p>" for item in unchecked_items]
+                )
+
+                # Load the unchecked items into the current editor
+                self._load_selected_date(False, unchecked_str)
+            else:
+                return False
+
+        except Exception as e:
+            raise SystemError(e)
+
     def _on_date_changed(self):
         """
         When the calendar selection changes, save the previous day's note if dirty,
@@ -515,6 +689,8 @@ class MainWindow(QMainWindow):
         self.cfg.path = new_cfg.path
         self.cfg.key = new_cfg.key
         self.cfg.idle_minutes = getattr(new_cfg, "idle_minutes", self.cfg.idle_minutes)
+        self.cfg.theme = getattr(new_cfg, "theme", self.cfg.theme)
+        self.cfg.move_todos = getattr(new_cfg, "move_todos", self.cfg.move_todos)
 
         # Persist once
         save_db_config(self.cfg)
@@ -564,7 +740,7 @@ class MainWindow(QMainWindow):
             QGuiApplication.screenAt(QCursor.pos()) or QGuiApplication.primaryScreen()
         )
         r = screen.availableGeometry()
-        # Center the window in that screen’s available area
+        # Center the window in that screen's available area
         self.move(r.center() - self.rect().center())
 
     # ----------------- Export handler ----------------- #
@@ -593,6 +769,7 @@ If you want an encrypted backup, choose Backup instead of Export.
             "JSON (*.json);;"
             "CSV (*.csv);;"
             "HTML (*.html);;"
+            "Markdown (*.md);;"
             "SQL (*.sql);;"
         )
 
@@ -608,6 +785,7 @@ If you want an encrypted backup, choose Backup instead of Export.
             "JSON (*.json)": ".json",
             "CSV (*.csv)": ".csv",
             "HTML (*.html)": ".html",
+            "Markdown (*.md)": ".md",
             "SQL (*.sql)": ".sql",
         }.get(selected_filter, ".txt")
 
@@ -624,6 +802,8 @@ If you want an encrypted backup, choose Backup instead of Export.
                 self.db.export_csv(entries, filename)
             elif selected_filter.startswith("HTML"):
                 self.db.export_html(entries, filename)
+            elif selected_filter.startswith("Markdown"):
+                self.db.export_markdown(entries, filename)
             elif selected_filter.startswith("SQL"):
                 self.db.export_sql(filename)
             else:
@@ -689,7 +869,7 @@ If you want an encrypted backup, choose Backup instead of Export.
             return
         if minutes == 0:
             self._idle_timer.stop()
-            # If you’re currently locked, unlock when user disables the timer:
+            # If currently locked, unlock when user disables the timer:
             if getattr(self, "_locked", False):
                 try:
                     self._locked = False
@@ -705,9 +885,14 @@ If you want an encrypted backup, choose Backup instead of Export.
     def eventFilter(self, obj, event):
         if event.type() == QEvent.KeyPress and not self._locked:
             self._idle_timer.start()
+        if event.type() in (QEvent.ApplicationActivate, QEvent.WindowActivate):
+            QTimer.singleShot(0, self._focus_editor_now)
         return super().eventFilter(obj, event)
 
     def _enter_lock(self):
+        """
+        Trigger the lock overlay and disable widgets
+        """
         if self._locked:
             return
         self._locked = True
@@ -723,6 +908,10 @@ If you want an encrypted backup, choose Backup instead of Export.
 
     @Slot()
     def _on_unlock_clicked(self):
+        """
+        Prompt for key to unlock screen
+        If successful, re-enable widgets
+        """
         try:
             ok = self._prompt_for_key_until_valid(first_time=False)
         except Exception as e:
@@ -739,6 +928,7 @@ If you want an encrypted backup, choose Backup instead of Export.
             if tb:
                 tb.setEnabled(True)
             self._idle_timer.start()
+            QTimer.singleShot(0, self._focus_editor_now)
 
     # ----------------- Close handlers ----------------- #
     def closeEvent(self, event):
@@ -754,3 +944,65 @@ If you want an encrypted backup, choose Backup instead of Export.
         except Exception:
             pass
         super().closeEvent(event)
+
+    # ----------------- Below logic helps focus the editor ----------------- #
+
+    def _focus_editor_now(self):
+        """Give focus to the editor and ensure the caret is visible."""
+        if getattr(self, "_locked", False):
+            return
+        if not self.isActiveWindow():
+            return
+        # Belt-and-suspenders: do it now and once more on the next tick
+        self.editor.setFocus(Qt.ActiveWindowFocusReason)
+        self.editor.ensureCursorVisible()
+        QTimer.singleShot(
+            0,
+            lambda: (
+                self.editor.setFocus(Qt.ActiveWindowFocusReason),
+                self.editor.ensureCursorVisible(),
+            ),
+        )
+
+    def _on_app_state_changed(self, state):
+        # Called on macOS/Wayland/Windows when the whole app re-activates
+        if state == Qt.ApplicationActive and self.isActiveWindow():
+            QTimer.singleShot(0, self._focus_editor_now)
+
+    def changeEvent(self, ev):
+        # Called on some platforms when the window's activation state flips
+        super().changeEvent(ev)
+        if ev.type() == QEvent.ActivationChange and self.isActiveWindow():
+            QTimer.singleShot(0, self._focus_editor_now)
+
+    def _set_editor_html_preserve_view(self, html: str):
+        ed = self.editor
+
+        # Save caret/selection and scroll
+        cur = ed.textCursor()
+        old_pos, old_anchor = cur.position(), cur.anchor()
+        v = ed.verticalScrollBar().value()
+        h = ed.horizontalScrollBar().value()
+
+        # Only touch the doc if it actually changed
+        ed.blockSignals(True)
+        if ed.toHtml() != html:
+            ed.setHtml(html)
+        ed.blockSignals(False)
+
+        # Restore scroll first
+        ed.verticalScrollBar().setValue(v)
+        ed.horizontalScrollBar().setValue(h)
+
+        # Restore caret/selection
+        cur = ed.textCursor()
+        cur.setPosition(old_anchor)
+        mode = (
+            QTextCursor.KeepAnchor if old_anchor != old_pos else QTextCursor.MoveAnchor
+        )
+        cur.setPosition(old_pos, mode)
+        ed.setTextCursor(cur)
+
+        # Refresh highlights if the theme changed
+        if hasattr(self, "findBar"):
+            self.findBar.refresh()
