@@ -14,6 +14,7 @@ class ToolBar(QToolBar):
     headingRequested = Signal(int)
     bulletsRequested = Signal()
     numbersRequested = Signal()
+    checkboxesRequested = Signal()
     alignRequested = Signal(Qt.AlignmentFlag)
     historyRequested = Signal()
     insertImageRequested = Signal()
@@ -86,6 +87,9 @@ class ToolBar(QToolBar):
         self.actNumbers.setToolTip("Numbered list")
         self.actNumbers.setCheckable(True)
         self.actNumbers.triggered.connect(self.numbersRequested)
+        self.actCheckboxes = QAction("☐", self)
+        self.actCheckboxes.setToolTip("Toggle checkboxes")
+        self.actCheckboxes.triggered.connect(self.checkboxesRequested)
 
         # Images
         self.actInsertImg = QAction("Image", self)
@@ -136,6 +140,11 @@ class ToolBar(QToolBar):
         for a in (self.actAlignL, self.actAlignC, self.actAlignR):
             a.setActionGroup(self.grpAlign)
 
+        self.grpLists = QActionGroup(self)
+        self.grpLists.setExclusive(True)
+        for a in (self.actBullets, self.actNumbers, self.actCheckboxes):
+            a.setActionGroup(self.grpLists)
+
         # Add actions
         self.addActions(
             [
@@ -150,6 +159,7 @@ class ToolBar(QToolBar):
                 self.actNormal,
                 self.actBullets,
                 self.actNumbers,
+                self.actCheckboxes,
                 self.actInsertImg,
                 self.actAlignL,
                 self.actAlignC,
